function driver_MIMO_SPIE(Tw, W, ns, maniSet, inputAlg, inputDelta, isplotsignals, isplotalgs)
    r = rand();
    r = floor(r * 10000);
    r = 1;
    rand('state', r);
    randn('state', r);

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % References (for below formula):
    % (1) Slepian, D., "Some asymptotic expansions for prolate spheroidal
    %   wave functions," Journal of Mathematics and Physics 44(1),
    %   pp. 99 - 140, (1965)
    % (2) Landau, H. J., and Widom, H., "Eigenvalue distribution of time
    %   and frequency limiting," Journal of Mathematical Analysis and
    %   Applications 77, pp. 469 - 481, (1980)
    c = pi * W * (Tw / 2);
    nb = floor((2 * c) / pi + log2(c) / pi); 
    fprintf('\nNumber of PSWFs: %d\n', nb);
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    % Compute time samples
    Tw = Tw/2;
    dt = 1/(4*W);
    xt = -Tw:dt:Tw;
    
    % Number of methods
    inputAlg = sort(inputAlg);
    nMethods = length(inputAlg);
    tol = 1e-4;
    maxInnerItr = ns * nb;
    
    %% Manifold Parameters
    ManiParams.IsCheckParams = 0;
    ManiParams(1).name = 'Oblique';
    ManiParams(1).n = nb;
    ManiParams(1).m = ns;
    ManiParams(1).ParamSet = maniSet;

    if(ManiParams(1).ParamSet == 1)
        retract = sprintf('qfRetraction');
        vectTrans = sprintf('parallelization_LCnotsatisfied');
    elseif(ManiParams(1).ParamSet == 2)
        retract = sprintf('expMapping');
        vectTrans = sprintf('parallelTranslation_LCsatisfied');
    elseif(ManiParams(1).ParamSet == 3)
        retract = sprintf('qfRetraction');
        vectTrans = sprintf('parallelTranslation_LCnotsatisfied');
    elseif(ManiParams(1).ParamSet == 4)
        retract = sprintf('qfRetraction');
        vectTrans = sprintf('parallelTranslation_LCsatisfied');
    end

    % Make folder
    dir = sprintf('images/newcostfunc_%dsignals_nb%d_time%0.0d_band%d_avg', ns, nb, Tw, W); 
    mkdir(dir);
    
    % Compute zero lag pswf functions
    fn = sprintf('images/newcostfunc_%dsignals_nb%d_time%0.0d_band%d_avg/pswf_func_time%0.0d_band%d_nb%d.mat', ns, nb, Tw, W, Tw, W, nb);
    if(exist(fn, 'file'))
        load(fn)
    else
        p0 = pswf(xt/Tw, Tw, W, nb).';
        save(fn, 'p0');
    end

    % Compute correlation matrix
    flag = sprintf('images/newcostfunc_%dsignals_nb%d_time%0.0d_band%d_avg/lags_time%0.0d_band%d_nb%d.mat', ns, nb, Tw, W, Tw, W, nb);
    if( exist(flag, 'file'))
        load(flag);
    else
        lag = -Tw:dt:Tw;
        save(flag, 'lag');
    end
    fprintf('lag = %f : dt : %f\n', -Tw, Tw);

    maxLag = max(lag)/dt;
    pad = ceil(maxLag);
    p0new = [p0; zeros(pad, nb)];

    nlags = length(lag);
    C = zeros(nb, nb, nlags);
    for l = 1 : nlags
        pl = fourierShift(p0new.', lag(l), dt).';
        C(:, :, l) = p0new' * pl;
    end

    % Initial Condtions
    fx0 = sprintf('images/newcostfunc_%dsignals_nb%d_time%0.0d_band%d_avg/initialCondition_time%0.0d_band%d_nb%d_ns%d.mat', ns, nb, Tw, W, Tw, W, nb, ns);
    if exist(fx0, 'file')
        load(fx0);
    else
        x0 = randn(nb, ns);
        [x0, ~] = qr(x0, 0);
        save(fx0, 'x0');
    end
    
    % Run for delta value(s)
    for delta = inputDelta
        fprintf('\nDelta_n: %d \n', delta);

        % Make Folder
        newDir = sprintf('delta%d/%s/%s/', delta, retract, vectTrans);
        currDir = sprintf('images/newcostfunc_%dsignals_nb%d_time%0.0d_band%d_avg', ns, nb, Tw, W); 
        mkdir(currDir, newDir);

        InitialX.main = x0;
        HasHHR = 0;
        fhandle = @(x)f(x, C, lag, delta);
        gfhandle = @(x)gf(x, C, lag, delta);
        Hesshandle = @(x, eta)Hess(x, eta, C, lag, delta);
        %% Solver Parameters
        
        tab = zeros(10, 7);
        for method = inputAlg
            if(method == 1)
                alg = sprintf('RBFGS');
                plotAlgName = sprintf('RBFGS');

                SolverParams.method = 'RBFGS';
                SolverParams.IsCheckParams = 0;
                SolverParams.IsCheckGradHess = 0;
                SolverParams.Stop_Criterion = 2;
                SolverParams.Tolerance = tol;
                SolverParams.Min_Iteration = 0;
                SolverParams.Max_Iteration = 5000;
                SolverParams.OutputGap = 1;
                SolverParams.DEBUG = 2;
                SolverParams.LineSearch_LS = 0;
                SolverParams.LS_alpha = 1e-4;
                SolverParams.LS_beta = 0.999;
                SolverParams.Minstepsize = eps;
                SolverParams.Maxstepsize = 1000;
                SolverParams.LS_ratio = 0.25;
                SolverParams.nu = 1e-6;
                SolverParams.mu = 1;
            elseif(method == 2)
                alg = sprintf('LRBFGS_2');
                plotAlgName = sprintf('LRBFGS:2');

                SolverParams.method = 'LRBFGS';
                SolverParams.IsCheckParams = 0;
                SolverParams.IsCheckGradHess = 0;
                SolverParams.Stop_Criterion = 2;
                SolverParams.Tolerance = tol;
                SolverParams.Min_Iteration = 0;
                SolverParams.Max_Iteration = 5000;
                SolverParams.OutputGap = 1;
                SolverParams.DEBUG = 2;
                SolverParams.LineSearch_LS = 0;
                SolverParams.LS_alpha = 1e-4;
                SolverParams.LS_beta = 0.999;
                SolverParams.Minstepsize = eps;
                SolverParams.Maxstepsize = 1000;
                SolverParams.LS_ratio = 0.25;
                SolverParams.nu = 1e-6;
                SolverParams.mu = 1;
                SolverParams.LengthSY = 2;
            elseif(method == 3)
                alg = sprintf('LRBFGS_4');
                plotAlgName = sprintf('LRBFGS:4');

                SolverParams.method = 'LRBFGS';
                SolverParams.IsCheckParams = 0;
                SolverParams.IsCheckGradHess = 0;
                SolverParams.Stop_Criterion = 2;
                SolverParams.Tolerance = tol;
                SolverParams.Min_Iteration = 0;
                SolverParams.Max_Iteration = 5000;
                SolverParams.OutputGap = 1;
                SolverParams.DEBUG = 2;
                SolverParams.LineSearch_LS = 0;
                SolverParams.LS_alpha = 1e-4;
                SolverParams.LS_beta = 0.999;
                SolverParams.Minstepsize = eps;
                SolverParams.Maxstepsize = 1000;
                SolverParams.LS_ratio = 0.25;
                SolverParams.nu = 1e-6;
                SolverParams.mu = 1;
                SolverParams.LengthSY = 4;

             elseif(method == 4)
                alg = sprintf('LRBFGS_8');
                plotAlgName = sprintf('LRBFGS:8');

                SolverParams.method = 'LRBFGS';
                SolverParams.IsCheckParams = 0;
                SolverParams.IsCheckGradHess = 0;
                SolverParams.Stop_Criterion = 2;
                SolverParams.Tolerance = tol;
                SolverParams.Min_Iteration = 0;
                SolverParams.Max_Iteration = 5000;
                SolverParams.OutputGap = 1;
                SolverParams.DEBUG = 2;
                SolverParams.LineSearch_LS = 0;
                SolverParams.LS_alpha = 1e-4;
                SolverParams.LS_beta = 0.999;
                SolverParams.Minstepsize = eps;
                SolverParams.Maxstepsize = 1000;
                SolverParams.LS_ratio = 0.25;
                SolverParams.nu = 1e-6;
                SolverParams.mu = 1;
                SolverParams.LengthSY = 8;
            elseif(method == 5)
                alg = sprintf('RSD');
                plotAlgName = sprintf('RSD');

                SolverParams.method = 'RSD';
                SolverParams.IsCheckParams = 0;
                SolverParams.IsCheckGradHess = 0;
                SolverParams.Stop_Criterion = 2;
                SolverParams.Tolerance = 1e-4;
                SolverParams.Min_Iteration = 0;
                SolverParams.Max_Iteration = 5000;
                SolverParams.OutputGap = 1;
                SolverParams.DEBUG = 2;
                SolverParams.LineSearch_LS = 0;
                SolverParams.LS_alpha = 1e-4;
                SolverParams.LS_beta = 0.999;
                SolverParams.Minstepsize = eps;
                SolverParams.Maxstepsize = 1000;
                SolverParams.LS_ratio = 0.25;
            elseif(method == 6)
                alg = sprintf('RNewton');
                plotAlgName = sprintf('RNewton');

                SolverParams.method = 'RNewton';
                SolverParams.IsCheckParams = 0;
                SolverParams.IsCheckGradHess = 0;
                SolverParams.Stop_Criterion = 2;
                SolverParams.Tolerance = tol;
                SolverParams.Min_Iteration = 0;
                SolverParams.Max_Iteration = 5000;
                SolverParams.OutputGap = 1;
                SolverParams.DEBUG = 2;
                SolverParams.LineSearch_LS = 0;
                SolverParams.LS_alpha = 1e-4;
                SolverParams.LS_beta = 0.999;
                SolverParams.Minstepsize = eps;
                SolverParams.Maxstepsize = 1000;
                SolverParams.LS_ratio = 0.25;
                SolverParams.useRand = 0;
                SolverParams.Min_Inner_Iter = 0;
                SolverParams.Max_Inner_Iter = maxInnerItr;
                SolverParams.theta = 1;
                SolverParams.kappa = 0.1;
            end
            currDir = sprintf('images/newcostfunc_%dsignals_nb%d_time%0.0d_band%d_avg/delta%d/%s/%s', ns, nb, Tw, W, delta, retract, vectTrans);
            mkdir(currDir, alg);

            %% Run Optimization Problem 
            times = 0;
            total_timecost = 0;
            T = 0;
            while(times < 5)
                X = {}; Fv = 0; GfG0 = 0; iter = 0; nf = 0; ng = 0; 
                nR = 0; nV = 0; nVp = 0; nH = 0; ComTime = 0;
                funs = []; grad = []; accumTimes = [];

                fprintf('%d: ', times);
                [X, Fv, Gfv, GfG0, iter, nf, ng, nR, nV, nVp, nH, ComTime, funs, grad, accumTimes] = DriverOPT(fhandle, gfhandle, Hesshandle, SolverParams, ManiParams, HasHHR, InitialX);
                if(times == 0)
                    T = accumTimes;
                else
                    T = T + accumTimes;
                end

                total_timecost = total_timecost + ComTime;
                times = times + 1;
            end
            T = T / times;
            ave_time = total_timecost / times;
            fprintf('\nAverage time cost of %d times is %f. \n', times, ave_time);

            tab(1, method + 1) = iter;
            tab(2, method + 1) = nf;
            tab(3, method + 1) = ng;
            tab(4, method + 1) = nH;
            tab(5, method + 1) = nV + nVp;
            tab(6, method + 1) = nR;
            tab(7, method + 1) = Fv;
            tab(8, method + 1) = Gfv;
            tab(9, method + 1) = GfG0;
            tab(10, method + 1) = ave_time;
            
            MIMO_FS{method} = funs;
            MIMO_GS{method} = grad;
            MIMO_GFG0S{method} = grad/grad(1);
            MIMO_TS{method} = T;

            S = p0 * X.main;
            [n, ~] = size(S);
            fb = sprintf('images/newcostfunc_%dsignals_nb%d_time%0.0d_band%d_avg/delta%d/%s/%s/%s/coef_time%0.0d_band%d_delta%d_nb%d.mat', ns, nb, Tw, W, delta, retract, vectTrans, alg, Tw, W, delta, nb);
            B = X.main;
            save(fb, 'B');
            fs = sprintf('images/newcostfunc_%dsignals_nb%d_time%0.0d_band%d_avg/delta%d/%s/%s/%s/signal_time%0.0d_band%d_delta%d_nb%d.mat', ns, nb, Tw, W, delta, retract, vectTrans, alg, Tw, W, delta, nb);
            save(fs, 'S');

            %% Signal and Auto- and Cross-Correlation Plots
            if(isplotsignals == 1)
                imgFilePng1 = sprintf('images/newcostfunc_%dsignals_nb%d_time%0.0d_band%d_avg/delta%d/%s/%s/%s/signals_time%0.0d_band%d_delta%d_nb%d.png', ns, nb, Tw, W, delta, retract, vectTrans, alg, Tw, W, delta, nb);
                imgFileFig1 = sprintf('images/newcostfunc_%dsignals_nb%d_time%0.0d_band%d_avg/delta%d/%s/%s/%s/signals_time%0.0d_band%d_delta%d_nb%d.fig', ns, nb, Tw, W, delta, retract, vectTrans, alg, Tw, W, delta, nb);
    %             imgFileTex1 = sprintf('images/newcostfunc_%dsignals_nb%d_time%0.0d_band%d_avg/delta%d/%s/%s/%s/signals_time%0.0d_band%d_delta%d_nb%d.tex', ns, nb, Tw, W, delta, retract, vectTrans, alg, Tw, W, delta, nb);
                if(ns == 2)
                    % Compare with LFM signals (only works with 2 signals)
                    sigTime = (1:n)*dt;
                    h = figure(1);clf
                    plot(sigTime, S(:, 1), 'b', sigTime, S(:, 2), '--r')
                    set(get(gcf,'CurrentAxes'),'fontsize',14)
                    xlabel('time', 'fontsize',14); 
                    legend('s_1', 's_2', 'Location', 'NorthEast');
                    axis([sigTime(1) sigTime(n) -0.4 0.4]);

                    saveas(h, imgFilePng1);
                    saveas(h, imgFileFig1);
    %                 cleanfigure;
    %                 matlab2tikz(imgFileTex1, 'showInfo', false, 'height', '\figureheight', 'width', '\figurewidth');
                    close(h)

                    % LFM Signals
                    h = figure(20);clf
                    [s1, s2] = mimo_lfm(2*Tw, W, dt);
                    [~, m1] = size(s1);
                    time1 = (1:m1)*dt;
                    plot(time1, s1, 'b', time1, s2, '--r')
                    set(get(gcf,'CurrentAxes'),'fontsize',14)
                    xlabel('time','fontsize',14); 
                    legend('s_1', 's_2', 'Location', 'NorthEast');
                    ylb = min(min(s1, s2));
                    yub = max(max(s1, s2));
                    axis([time1(1) time1(m1) ylb yub]);

                    imgFilePng20 = sprintf('images/newcostfunc_%dsignals_nb%d_time%0.0d_band%d_avg/delta%d/%s/%s/%s/LFMsignals_time%0.0d_band%d.png', ns, nb, Tw, W, delta, retract, vectTrans, alg, Tw, W);
                    imgFileFig20 = sprintf('images/newcostfunc_%dsignals_nb%d_time%0.0d_band%d_avg/delta%d/%s/%s/%s/LFMsignals_time%0.0d_band%d.fig', ns, nb, Tw, W, delta, retract, vectTrans, alg, Tw, W);
    %                 imgFileTex20 = sprintf('images/newcostfunc_%dsignals_nb%d_time%0.0d_band%d_avg/delta%d/%s/%s/%s/LFMsignals_time%0.0d_band%d.tex', ns, nb, Tw, W, delta, retract, vectTrans, alg, Tw, W);

                    saveas(h, imgFilePng20);
                    saveas(h, imgFileFig20);
    %                 cleanfigure;
    %                 matlab2tikz(imgFileTex20, 'showInfo', false, 'height', '\figureheight', 'width', '\figurewidth');
                    close(h);

                    for nsi = 1 : ns
                        imgFilePng2 = sprintf('images/newcostfunc_%dsignals_nb%d_time%0.0d_band%d_avg/delta%d/%s/%s/%s/auto_corr_s%d_time%0.0d_band%d_delta%d_nb%d.png', ns, nb, Tw, W, delta, retract, vectTrans, alg, nsi, Tw, W, delta, nb);
                        imgFileFig2 = sprintf('images/newcostfunc_%dsignals_nb%d_time%0.0d_band%d_avg/delta%d/%s/%s/%s/auto_corr_s%d_time%0.0d_band%d_delta%d_nb%d.fig', ns, nb, Tw, W, delta, retract, vectTrans, alg, nsi, Tw, W, delta, nb);
    %                     imgFileTex2 = sprintf('images/newcostfunc_%dsignals_nb%d_time%0.0d_band%d_avg/delta%d/%s/%s/%s/auto_corr_s%d_time%0.0d_band%d_delta%d_nb%d.tex', ns, nb, Tw, W, delta, retract, vectTrans, alg, nsi, Tw, W, delta, nb);
                        % Auto-Correltaion
                        h = figure(2);clf
                        hold on;
                        [acf2, aclags] = xcorr(S(:, nsi), floor(n/2));
                        plot(aclags*dt, 20*log10(abs(acf2)))
                        axis([-Tw Tw -30 0])
                        set(gca,'fontsize',14)

                        if(nsi == 1)
                            [acf2, aclags] = xcorr(s1, floor(m1/2));
                            plot(aclags*dt, 20*log10(abs(acf2)), '--r')
                        elseif(nsi == 2)
                            [acf2, aclags] = xcorr(s2, floor(m1/2));
                            plot(aclags*dt, 20*log10(abs(acf2)), '--r')
                        else
                            error('Cannot compute more that two LFM signals!');
                        end

                        imgTitle = sprintf('Auto-correlation of s_{%d}', nsi);
                        title(imgTitle,'fontsize',14)
                        xlabel('\tau time lag(s)','fontsize',14);
                        imgYLABEL = sprintf('20 log_{10}(| \\gamma_{%d,%d}( \\tau ) |)', nsi, nsi);
                        ylabel(imgYLABEL, 'fontsize',14);

                        autoCorr = sprintf('%s', plotAlgName);
                        legend(autoCorr, 'LFM', 'Location', 'NorthEast');
                        hold off;

                        saveas(h, imgFilePng2);
                        saveas(h, imgFileFig2);
    %                     cleanfigure;
    %                     matlab2tikz(imgFileTex2, 'showInfo', false, 'height', '\figureheight', 'width', '\figurewidth');
                        close(h)

                        for nsj = nsi+1 : ns
                            imgFilePng3 = sprintf('images/newcostfunc_%dsignals_nb%d_time%0.0d_band%d_avg/delta%d/%s/%s/%s/cross_corr_s%d_s%d_time%0.0d_band%d_delta%d_nb%d.png', ns, nb, Tw, W, delta, retract, vectTrans, alg, nsi, nsj, Tw, W, delta, nb);
                            imgFileFig3 = sprintf('images/newcostfunc_%dsignals_nb%d_time%0.0d_band%d_avg/delta%d/%s/%s/%s/cross_corr_s%d_s%d_time%0.0d_band%d_delta%d_nb%d.fig', ns, nb, Tw, W, delta, retract, vectTrans, alg, nsi, nsj, Tw, W, delta, nb);
    %                         imgFileTex3 = sprintf('images/newcostfunc_%dsignals_nb%d_time%0.0d_band%d_avg/delta%d/%s/%s/%s/cross_corr_s%d_s%d_time%0.0d_band%d_delta%d_nb%d.tex', ns, nb, Tw, W, delta, retract, vectTrans, alg, nsi, nsj, Tw, W, delta, nb);

                            % Cross-Correlation
                            h = figure(3);clf
                            hold on;
                            [ccf, clags] = xcorr(S(:, nsi), S(:, nsj), floor(n/2));
                            plot(clags*dt, 20*log10(abs(ccf)))
                            axis([-Tw Tw -100 0])
                            set(gca,'fontsize',14)

                            [ccf, clags] = xcorr(s1, s2, floor(m1/2));
                            plot(clags*dt, 20*log10(abs(ccf)), '--r')

                            imgTitle = sprintf('Cross-correlation between s_{%d} and s_{%d}', nsi, nsj);
                            title(imgTitle,'fontsize',14)
                            xlabel('\tau time lag(s)','fontsize',14);
                            imgYLABEL = sprintf('20 log_{10}(| \\gamma_{%d,%d}( \\tau ) |)', nsi, nsj);
                            ylabel(imgYLABEL, 'fontsize',14);
                            xCorrS1S2 = sprintf('%s', plotAlgName);
                            legend(xCorrS1S2, 'LFM', 'Location', 'NorthEast');
                            hold off;

                            saveas(h, imgFilePng3);
                            saveas(h, imgFileFig3);
    %                         cleanfigure;
    %                         matlab2tikz(imgFileTex3, 'showInfo', false, 'height', '\figureheight', 'width', '\figurewidth');
                            close(h)
                        end
                    end
                else
                    % More than 2 signals
                    sigTime = (1:n)*dt;
                    h = figure(1);clf

                    col = num2cell(jet(ns), 2);
                    markers = {'--o', '--x', '--s', '--^', '--*', '--+', '--v', '--d'};
                    for i = 1 : ns
                        p = plot(sigTime, S(:, i), markers{i});
                        set(p, 'color' ,col{i})
                        hold on;
                    end
                    xlabel('time', 'fontsize', 14); 
                    axis([sigTime(1) sigTime(n) -0.4 0.4]);
                    set(gca,'fontsize',14)
                    
                    sLegend = cell(ns, 1);
                    for nsi = 1 : ns
                        sLegend{nsi} = sprintf('s_{%d}', nsi);
                    end
                    legend(sLegend, 'Location', 'NorthEastOutside');
                    hold off;

                    saveas(h, imgFilePng1);
                    saveas(h, imgFileFig1);
    %                 cleanfigure;
    %                 matlab2tikz(imgFileTex1, 'showInfo', false, 'height', '\figureheight', 'width', '\figurewidth');
                    close(h)

                    for nsi = 1 : ns
                        imgFilePng2 = sprintf('images/newcostfunc_%dsignals_nb%d_time%0.0d_band%d_avg/delta%d/%s/%s/%s/auto_corr_s%d_time%0.0d_band%d_delta%d_nb%d.png', ns, nb, Tw, W, delta, retract, vectTrans, alg, nsi, Tw, W, delta, nb);
                        imgFileFig2 = sprintf('images/newcostfunc_%dsignals_nb%d_time%0.0d_band%d_avg/delta%d/%s/%s/%s/auto_corr_s%d_time%0.0d_band%d_delta%d_nb%d.fig', ns, nb, Tw, W, delta, retract, vectTrans, alg, nsi, Tw, W, delta, nb);
    %                     imgFileTex2 = sprintf('images/newcostfunc_%dsignals_nb%d_time%0.0d_band%d_avg/delta%d/%s/%s/%s/auto_corr_s%d_time%0.0d_band%d_delta%d_nb%d.tex', ns, nb, Tw, W, delta, retract, vectTrans, alg, nsi, Tw, W, delta, nb);
                        % Auto-Correltaion
                        h = figure('Visible', 'off');clf
                        hold on;
                        [acf2, aclags] = xcorr(S(:, nsi), floor(n/2));
                        plot(aclags*dt, 20*log10(abs(acf2)))
                        axis([-Tw Tw -30 0])
                        set(gca,'fontsize',14)

                        imgTitle = sprintf('Auto-correlation of s_{%d}', nsi);
                        title(imgTitle,'fontsize',14)
                        xlabel('\tau time lag(s)','fontsize',14);
                        imgYLABEL = sprintf('20 log_{10}(| \\gamma_{%d,%d}( \\tau ) |)', nsi, nsi);
                        ylabel(imgYLABEL, 'fontsize',14);
                        autoCorr = sprintf('%s', plotAlgName);
                        legend(autoCorr, 'Location', 'NorthEast');
                        hold off;

                        saveas(h, imgFilePng2);
                        saveas(h, imgFileFig2);
    %                     cleanfigure;
    %                     matlab2tikz(imgFileTex2, 'showInfo', false, 'height', '\figureheight', 'width', '\figurewidth');
                        close(h)

                        for nsj = nsi+1 : ns
                            imgFilePng3 = sprintf('images/newcostfunc_%dsignals_nb%d_time%0.0d_band%d_avg/delta%d/%s/%s/%s/cross_corr_s%d_s%d_time%0.0d_band%d_delta%d_nb%d.png', ns, nb, Tw, W, delta, retract, vectTrans, alg, nsi, nsj, Tw, W, delta, nb);
                            imgFileFig3 = sprintf('images/newcostfunc_%dsignals_nb%d_time%0.0d_band%d_avg/delta%d/%s/%s/%s/cross_corr_s%d_s%d_time%0.0d_band%d_delta%d_nb%d.fig', ns, nb, Tw, W, delta, retract, vectTrans, alg, nsi, nsj, Tw, W, delta, nb);
    %                         imgFileTex3 = sprintf('images/newcostfunc_%dsignals_nb%d_time%0.0d_band%d_avg/delta%d/%s/%s/%s/cross_corr_s%d_s%d_time%0.0d_band%d_delta%d_nb%d.tex', ns, nb, Tw, W, delta, retract, vectTrans, alg, nsi, nsj, Tw, W, delta, nb);

                            % Cross-Correlation
                            h = figure('Visible', 'off');clf
                            hold on;
                            [ccf, clags] = xcorr(S(:, nsi), S(:, nsj), floor(n/2));
                            plot(clags*dt, 20*log10(abs(ccf)))
                            axis([-Tw Tw -100 0])
                            set(gca,'fontsize',14)

                            imgTitle = sprintf('Cross-correlation between s_{%d} and s_{%d}', nsi, nsj);
                            title(imgTitle,'fontsize',14)
                            xlabel('\tau time lag(s)','fontsize',14);
                            imgYLABEL = sprintf('20 log_{10}(| \\gamma_{%d,%d}( \\tau ) |)', nsi, nsj);
                            ylabel(imgYLABEL, 'fontsize',14);
                            xCorrS1S2 = sprintf('%s', plotAlgName);
                            legend(xCorrS1S2, 'Location', 'NorthEast');
                            hold off;

                            saveas(h, imgFilePng3);
                            saveas(h, imgFileFig3);
    %                         cleanfigure;
    %                         matlab2tikz(imgFileTex3, 'showInfo', false, 'height', '\figureheight', 'width', '\figurewidth');
                            close(h)
                        end
                    end
                end
            end
        end

        %% Save Table of Algorithm Comparison
        algLegend = cell(nMethods, 1);
        algName = cell(nMethods, 1);
        for ni = 1 : nMethods
            if(inputAlg(ni) == 1)
                alg = sprintf('RBFGS');
                algName{ni} = alg;
            elseif(inputAlg(ni) == 2)
                alg = sprintf('LRBFGS:2');
                algName{ni} = sprintf('LRBFGS2');
            elseif(inputAlg(ni) == 3)
                alg = sprintf('LRBFGS:4');
                algName{ni} = sprintf('LRBFGS4');
            elseif(inputAlg(ni) == 4)
                alg = sprintf('LRBFGS:8');
                algName{ni} = sprintf('LRBFGS8');
            elseif(inputAlg(ni) == 5)
                alg = sprintf('RSD');
                algName{ni} = alg;
            elseif(inputAlg(ni) == 6)
                alg = sprintf('RNewton');
                algName{ni} = alg;
            end
            algLegend{ni} = alg;
        end
        strAlg = strjoin(algName,'_');
        
        tmpJ = [1, (inputAlg + 1)];
        fileName = sprintf('images/newcostfunc_%dsignals_nb%d_time%0.0d_band%d_avg/delta%d/%s/%s/MIMO_%s_tab.txt', ns, nb, Tw, W, delta, retract, vectTrans, strAlg);
        fout = fopen(fileName,'w+');
        for i = 1 : 10
            for j = tmpJ
                if(j == 1)
                    if(i == 1)
                        tabstr = sprintf('iter');
                    elseif(i == 2)
                        tabstr = sprintf('nf');
                    elseif(i == 3)
                        tabstr = sprintf('ng');
                    elseif(i == 4)
                        tabstr = sprintf('nH');
                    elseif(i == 5)
                        tabstr = sprintf('nV');
                    elseif(i == 6)
                        tabstr = sprintf('nR');
                    elseif(i == 7)
                        tabstr = sprintf('f_{f}');
                    elseif(i == 8)
                        tabstr = sprintf('gf_{f}');
                    elseif(i == 9)
                        tabstr = sprintf('gf_{f}/gf_{0}');
                    else
                        tabstr = sprintf('t (sec)');
                    end
                    fprintf(fout,['$' tabstr '$ & ']);
                else
                    if(mod(i-1, 10) < 6)
                        if(j < 7)
                            fprintf(fout,'%d & ',tab(i, j));
                        else
                            fprintf(fout,'%d', tab(i, j));
                        end
                    else
                        tabstr = outputfloat(tab(i, j));
                        if(j < 7)
                            fprintf(fout,['$' tabstr '$ & ']);
                        else
                            fprintf(fout,['$' tabstr '$']);
                        end
                    end
                end
            end
            fprintf(fout,' \\\\\n');
        end
        fclose(fout);

        fMIMO_FS = sprintf('images/newcostfunc_%dsignals_nb%d_time%0.0d_band%d_avg/delta%d/%s/%s/algs_%s_functionVals_time%0.0d_band%d_delta%d_nb%d.mat', ns, nb, Tw, W, delta, retract, vectTrans, strAlg, Tw, W, delta, nb);
        fMIMO_GS = sprintf('images/newcostfunc_%dsignals_nb%d_time%0.0d_band%d_avg/delta%d/%s/%s/algs_%s_normGradientVals_time%0.0d_band%d_delta%d_nb%d.mat', ns, nb, Tw, W, delta, retract, vectTrans, strAlg, Tw, W, delta, nb);
        fMIMO_GFG0S = sprintf('images/newcostfunc_%dsignals_nb%d_time%0.0d_band%d_avg/delta%d/%s/%s/algs_%s_normGradfGrad0Vals_time%0.0d_band%d_delta%d_nb%d.mat', ns, nb, Tw, W, delta, retract, vectTrans, strAlg, Tw, W, delta, nb);
        fMIMO_TS = sprintf('images/newcostfunc_%dsignals_nb%d_time%0.0d_band%d_avg/delta%d/%s/%s/algs_%s_timeVals_time%0.0d_band%d_delta%d_nb%d.mat', ns, nb, Tw, W, delta, retract, vectTrans, strAlg, Tw, W, delta, nb);
        save(fMIMO_FS, 'MIMO_FS');
        save(fMIMO_GS, 'MIMO_GS');
        save(fMIMO_GFG0S, 'MIMO_GFG0S');
        save(fMIMO_TS, 'MIMO_TS');

        %% Plot Algorithm Comparison
        if(isplotalgs == 1)
            % Gradient: Algorithm Iteration and Time Comparison
            h = figure('Position', [1, 1, 720, 480], 'Visible', 'off');clf
            set(0,'defaultaxesfontsize',10, ...
               'defaultaxeslinewidth',0.7, ...
               'defaultlinelinewidth',.8,'defaultpatchlinewidth',0.8);
            set(0,'defaultlinemarkersize',10)
            col = num2cell(jet(6), 2);
            markers = {'--o', '--x', '--s', '--*', '--+', '--d'};

            for i = inputAlg
                p = semilogy(1 : length(MIMO_GS{i}), MIMO_GS{i}, markers{i});
                set(p, 'color' ,col{i});
                hold on
            end
            axis square
            axis fill
            axis tight
            xlabel('iter')
            ylabel('|grad f|')
            legend(algLegend, 'Location', 'NorthEastOutside');

            algFilePng = sprintf('images/newcostfunc_%dsignals_nb%d_time%0.0d_band%d_avg/delta%d/%s/%s/algs_%s_grad_iter_time%0.0d_band%d_delta%d_nb%d.png', ns, nb, Tw, W, delta, retract, vectTrans, strAlg, Tw, W, delta, nb);
            algFileFig = sprintf('images/newcostfunc_%dsignals_nb%d_time%0.0d_band%d_avg/delta%d/%s/%s/algs_%s_grad_iter_time%0.0d_band%d_delta%d_nb%d.fig', ns, nb, Tw, W, delta, retract, vectTrans, strAlg, Tw, W, delta, nb);
    %         algFileTex = sprintf('images/newcostfunc_%dsignals_nb%d_time%0.0d_band%d_avg/delta%d/%s/%s/algs_%s_grad_iter_time%0.0d_band%d_delta%d_nb%d.tex', ns, nb, Tw, W, delta, retract, vectTrans, strAlg, Tw, W, delta, nb);
            saveas(h, algFilePng);
            saveas(h, algFileFig);
    %         axoptions={'ylabel={$\lvert$ grad f $\rvert$}'}; 
    %         cleanfigure;
    %         matlab2tikz(algFileTex, 'showInfo', false, 'height', '\figureheight', 'width', '\figurewidth', 'extraAxisOptions', axoptions);
            close(h)


            h = figure('Position', [1, 1, 720, 480], 'Visible', 'off');clf
            set(0,'defaultaxesfontsize',10, ...
               'defaultaxeslinewidth',0.7, ...
               'defaultlinelinewidth',.8,'defaultpatchlinewidth',0.8);
            set(0,'defaultlinemarkersize',10)
            col = num2cell(jet(6), 2);
            markers = {'--o', '--x', '--s', '--*', '--+', '--d'};

            for i = inputAlg
                p = semilogy(MIMO_TS{i}, MIMO_GS{i}, markers{i});
                set(p, 'color' ,col{i})
                hold on
            end
            axis square
            axis fill
            axis tight
            xlabel('time (second)')
            ylabel('|grad f|')
            legend(algLegend, 'Location', 'NorthEastOutside');

            algFilePng = sprintf('images/newcostfunc_%dsignals_nb%d_time%0.0d_band%d_avg/delta%d/%s/%s/algs_%s_grad_time_time%0.0d_band%d_delta%d_nb%d.png', ns, nb, Tw, W, delta, retract, vectTrans, strAlg, Tw, W, delta, nb);
            algFileFig = sprintf('images/newcostfunc_%dsignals_nb%d_time%0.0d_band%d_avg/delta%d/%s/%s/algs_%s_grad_time_time%0.0d_band%d_delta%d_nb%d.fig', ns, nb, Tw, W, delta, retract, vectTrans, strAlg, Tw, W, delta, nb);
    %         algFileTex = sprintf('images/newcostfunc_%dsignals_nb%d_time%0.0d_band%d_avg/delta%d/%s/%s/algs_%s_grad_time_time%0.0d_band%d_delta%d_nb%d.tex', ns, nb, Tw, W, delta, retract, vectTrans, strAlg, Tw, W, delta, nb);        
            saveas(h, algFilePng);
            saveas(h, algFileFig);
    %         axoptions={'ylabel={$\lvert$ grad f $\rvert$}'}; 
    %         cleanfigure;
    %         matlab2tikz(algFileTex, 'showInfo', false, 'height', '\figureheight', 'width', '\figurewidth', 'extraAxisOptions', axoptions);
            close(h)

            % Gradient: Algorithm Iteration and Time Comparison
            h = figure('Position', [1, 1, 720, 480], 'Visible', 'off');clf
            set(0,'defaultaxesfontsize',10, ...
               'defaultaxeslinewidth',0.7, ...
               'defaultlinelinewidth',.8,'defaultpatchlinewidth',0.8);
            set(0,'defaultlinemarkersize',10)
            col = num2cell(jet(6), 2);
            markers = {'--o', '--x', '--s', '--*', '--+', '--d'};

            for i = inputAlg
                p = semilogy(1 : length(MIMO_GFG0S{i}), MIMO_GFG0S{i}, markers{i});
                set(p, 'color' ,col{i});
                hold on
            end
            axis square
            axis fill
            axis tight
            xlabel('iter')
            ylabel('|grad f| / |grad f_0|')
            legend(algLegend, 'Location', 'NorthEastOutside');

            algFilePng = sprintf('images/newcostfunc_%dsignals_nb%d_time%0.0d_band%d_avg/delta%d/%s/%s/algs_%s_gradfgrad0_iter_time%0.0d_band%d_delta%d_nb%d.png', ns, nb, Tw, W, delta, retract, vectTrans, strAlg, Tw, W, delta, nb);
            algFileFig = sprintf('images/newcostfunc_%dsignals_nb%d_time%0.0d_band%d_avg/delta%d/%s/%s/algs_%s_gradfgrad0_iter_time%0.0d_band%d_delta%d_nb%d.fig', ns, nb, Tw, W, delta, retract, vectTrans, strAlg, Tw, W, delta, nb);
    %         algFileTex = sprintf('images/newcostfunc_%dsignals_nb%d_time%0.0d_band%d_avg/delta%d/%s/%s/algs_%s_gradfgrad0_iter_time%0.0d_band%d_delta%d_nb%d.tex', ns, nb, Tw, W, delta, retract, vectTrans, strAlg, Tw, W, delta, nb);
            saveas(h, algFilePng);
            saveas(h, algFileFig);
    %         axoptions={'ylabel={$\frac{\lvert\text{ grad f }\rvert}{\lvert\text{ grad f$_0$ }\rvert}$}'}; 
    %         cleanfigure;
    %         matlab2tikz(algFileTex, 'showInfo', false, 'height', '\figureheight', 'width', '\figurewidth', 'extraAxisOptions', axoptions);
            close(h)


            h = figure('Position', [1, 1, 720, 480], 'Visible', 'off');clf
            set(0,'defaultaxesfontsize',10, ...
               'defaultaxeslinewidth',0.7, ...
               'defaultlinelinewidth',.8,'defaultpatchlinewidth',0.8);
            set(0,'defaultlinemarkersize',10)
            col = num2cell(jet(6), 2);
            markers = {'--o', '--x', '--s', '--*', '--+', '--d'};

            for i = inputAlg
                p = semilogy(MIMO_TS{i}, MIMO_GFG0S{i}, markers{i});
                set(p, 'color' ,col{i})
                hold on
            end
            axis square
            axis fill
            axis tight
            xlabel('time (second)')
            ylabel('|grad f| / |grad f_0|')
            legend(algLegend, 'Location', 'NorthEastOutside');

            algFilePng = sprintf('images/newcostfunc_%dsignals_nb%d_time%0.0d_band%d_avg/delta%d/%s/%s/algs_%s_gradfgrad0_time_time%0.0d_band%d_delta%d_nb%d.png', ns, nb, Tw, W, delta, retract, vectTrans, strAlg, Tw, W, delta, nb);
            algFileFig = sprintf('images/newcostfunc_%dsignals_nb%d_time%0.0d_band%d_avg/delta%d/%s/%s/algs_%s_gradfgrad0_time_time%0.0d_band%d_delta%d_nb%d.fig', ns, nb, Tw, W, delta, retract, vectTrans, strAlg, Tw, W, delta, nb);
    %         algFileTex = sprintf('images/newcostfunc_%dsignals_nb%d_time%0.0d_band%d_avg/delta%d/%s/%s/algs_%s_gradfgrad0_time_time%0.0d_band%d_delta%d_nb%d.tex', ns, nb, Tw, W, delta, retract, vectTrans, strAlg, Tw, W, delta, nb);        
            saveas(h, algFilePng);
            saveas(h, algFileFig);
    %         axoptions={'ylabel={$\frac{\lvert\text{ grad f }\rvert}{\lvert\text{ grad f$_0$ }\rvert}$}'};
    %         cleanfigure;
    %         matlab2tikz(algFileTex, 'showInfo', false, 'height', '\figureheight', 'width', '\figurewidth', 'extraAxisOptions', axoptions);
            close(h)

            % Function: Algorithm Iteration and Time Comparison
            h = figure('Position', [1, 1, 720, 480], 'Visible', 'off');clf
            set(0,'defaultaxesfontsize',10, ...
               'defaultaxeslinewidth',0.7, ...
               'defaultlinelinewidth',.8,'defaultpatchlinewidth',0.8);
            set(0,'defaultlinemarkersize',10)
            col = num2cell(jet(6), 2);
            markers = {'--o', '--x', '--s', '--*', '--+', '--d'};

            for i = inputAlg
                p = semilogy(1 : length(MIMO_FS{i}), MIMO_FS{i}, markers{i});
                set(p, 'color' ,col{i});
                hold on
            end
            axis square
            axis fill
            axis tight
            xlabel('iter')
            ylabel('f')
            legend(algLegend, 'Location', 'NorthEastOutside');

            algFilePng = sprintf('images/newcostfunc_%dsignals_nb%d_time%0.0d_band%d_avg/delta%d/%s/%s/algs_%s_func_iter_time%0.0d_band%d_delta%d_nb%d.png', ns, nb, Tw, W, delta, retract, vectTrans, strAlg, Tw, W, delta, nb);
            algFileFig = sprintf('images/newcostfunc_%dsignals_nb%d_time%0.0d_band%d_avg/delta%d/%s/%s/algs_%s_func_iter_time%0.0d_band%d_delta%d_nb%d.fig', ns, nb, Tw, W, delta, retract, vectTrans, strAlg, Tw, W, delta, nb);
    %         algFileTex = sprintf('images/newcostfunc_%dsignals_nb%d_time%0.0d_band%d_avg/delta%d/%s/%s/algs_%s_func_iter_time%0.0d_band%d_delta%d_nb%d.tex', ns, nb, Tw, W, delta, retract, vectTrans, strAlg, Tw, W, delta, nb);
            saveas(h, algFilePng);
            saveas(h, algFileFig);
    %         cleanfigure;
    %         matlab2tikz(algFileTex, 'showInfo', false, 'height', '\figureheight', 'width', '\figurewidth');
            close(h)


            h = figure('Position', [1, 1, 720, 480], 'Visible', 'off');clf
            set(0,'defaultaxesfontsize',10, ...
               'defaultaxeslinewidth',0.7, ...
               'defaultlinelinewidth',.8,'defaultpatchlinewidth',0.8);
            set(0,'defaultlinemarkersize',10)
            col = num2cell(jet(6), 2);
            markers = {'--o', '--x', '--s', '--*', '--+', '--d'};

            for i = inputAlg
                p = semilogy(MIMO_TS{i}, MIMO_FS{i}, markers{i});
                set(p, 'color' ,col{i})
                hold on
            end
            axis square
            axis fill
            axis tight
            xlabel('time (second)')
            ylabel('f')
            legend(algLegend, 'Location', 'NorthEastOutside');

            algFilePng = sprintf('images/newcostfunc_%dsignals_nb%d_time%0.0d_band%d_avg/delta%d/%s/%s/algs_%s_func_time_time%0.0d_band%d_delta%d_nb%d.png', ns, nb, Tw, W, delta, retract, vectTrans, strAlg, Tw, W, delta, nb);
            algFileFig = sprintf('images/newcostfunc_%dsignals_nb%d_time%0.0d_band%d_avg/delta%d/%s/%s/algs_%s_func_time_time%0.0d_band%d_delta%d_nb%d.fig', ns, nb, Tw, W, delta, retract, vectTrans, strAlg, Tw, W, delta, nb);
    %         algFileTex = sprintf('images/newcostfunc_%dsignals_nb%d_time%0.0d_band%d_avg/delta%d/%s/%s/algs_%s_func_time_time%0.0d_band%d_delta%d_nb%d.tex', ns, nb, Tw, W, delta, retract, vectTrans, strAlg, Tw, W, delta, nb);        
            saveas(h, algFilePng);
            saveas(h, algFileFig);
    %         cleanfigure;
    %         matlab2tikz(algFileTex, 'showInfo', false, 'height', '\figureheight', 'width', '\figurewidth');
            close(h)
        end
    end
 end

%% 
function str = outputfloat(x)
    if(x <= 0)
        str = '';
        return;
    end
    p = log(x)/log(10);
    p = - ceil(-p);
    x = round(x * 10^(-p) * 100);
    x = x / 100;
    strx = sprintf('%3.2f', x);
    if(p ~= 0)
        str = [strx '_{' num2str(p) '}'];
    else
        str = strx;
    end
end

%% Euclidean Cost Function, Gradient and Hessian
function [output, x] = f(x, C, lag, delta)
    % Cost Function
%     [nb,  ns] = size(x.main);
    N = size(C, 3);
    I1 = 0;
    
    % Neighborhood around tau = 0
    dl = ceil(N/2)-delta;
    du = ceil(N/2)+delta;
    
    a = lag(1); b = lag(dl);
    N1 = length(1 : dl);
    for i = 1 : dl
        tmp1 = C(:, :, i) * x.main;
        tmp2 = x.main' * tmp1;
        f = norm(tmp2,'fro')^2;
        
        if(i == 1 || i == dl)
            I1 = I1 + ((b - a) / (N1 - 1)) * (f / 2);
        else
            I1 = I1 + ((b - a) / (N1 - 1)) * f;
        end
    end
    
    a = lag(dl); b = lag(du);
    N2 = length(dl : du);
    for i = dl : du
        tmp1 = C(:, :, i) * x.main;
        tmp2 = x.main' * tmp1;
        
        fO = norm((tmp2 - diag(diag(tmp2))),'fro')^2;
        
        if(i == dl || i == du)
            I1 = I1 + ((b - a) / (N2 - 1)) * (fO / 2);
        else
            I1 = I1 + ((b - a) / (N2 - 1)) * fO;
        end
    end
    
    a = lag(du); b = lag(N);
    N3 = length(du : N);
    for i = du : N
        tmp1 = C(:, :, i) * x.main;
        tmp2 = x.main' * tmp1;
        f = norm(tmp2,'fro')^2;
        
        if(i == du || i == N)
            I1 = I1 + ((b - a) / (N3 - 1)) * (f / 2);
        else
            I1 = I1 + ((b - a) / (N3 - 1)) * f;
        end
    end
    output = I1;
end

function [output, x] = gf(x, C, lag, delta)
    % Euclidean Gradient
    [nb, ns] = size(x.main);
    N = size(C, 3);
    Q1 = zeros(nb, ns);
    
    dl = ceil(N/2)-delta;
    du = ceil(N/2)+delta;
    
    a = lag(1); b = lag(dl);
    N1 = length(1 : dl);
    for i = 1 : dl
        tmp1 = C(:, :, i) * x.main;
        tmp2 = x.main' * tmp1;
        tmp3 = C(:, :, i)' * x.main;
        gf = (tmp1 * tmp2') + (tmp3 * tmp2);
        
        if(i == 1 || i == dl)
            Q1 = Q1 + ((b - a) / (N1 - 1)) * (gf / 2);
        else
            Q1 = Q1 + ((b - a) / (N1 - 1)) * gf;
        end
    end
    
    a = lag(dl); b = lag(du);
    N2 = length(dl : du);
    for i = dl : du
        tmp1 = C(:, :, i) * x.main;
        tmp2 = x.main' * tmp1;
        tmp3 = C(:, :, i)' * x.main;
        
        offDiag = tmp2 - diag(diag(tmp2));
        
        gfO = (tmp1 * offDiag') + (tmp3 * offDiag);
%         gfO = (tmp1 * (triu(tmp2 ,1) + tril(tmp2 ,-1))') + (tmp3 * (triu(tmp2,1) + tril(tmp2,-1)));
        
        if(i == dl || i == du)
            Q1 = Q1 + ((b - a) / (N2 - 1)) * (gfO / 2);
        else
            Q1 = Q1 + ((b - a) / (N2 - 1)) * gfO;
        end
    end
    
    a = lag(du); b = lag(N);
    N3 = length(du : N);
    for i = du : N
        tmp1 = C(:, :, i) * x.main;
        tmp2 = x.main' * tmp1;
        tmp3 = C(:, :, i)' * x.main;
        gf = (tmp1 * tmp2') + (tmp3 * tmp2);
        
        if(i == du || i == N)
            Q1 = Q1 + ((b - a) / (N3 - 1)) * (gf / 2);
        else
            Q1 = Q1 + ((b - a) / (N3 - 1)) * gf;
        end
    end

    output.main = 2 * Q1;
end

function [output, x] = Hess(x, eta, C, lag, delta)
    % Euclidean Hessian
    [nb, ns] = size(x.main);
    N = size(C, 3);
    Q2 = zeros(nb, ns);

    dl = ceil(N/2)-delta;
    du = ceil(N/2)+delta;
    
    a = lag(1); b = lag(dl);
    N1 = length(1 : dl);
    for i = 1 : dl
        tmp1 = C(:, :, i) * x.main;
        tmp2 = x.main' * tmp1;
        tmp3 = C(:, :, i)' * x.main;
        tmp4 = C(:, :, i) * eta.main;
        tmp5 = C(:, :, i)' * eta.main;
        tmp6 = eta.main' * tmp1;
        tmp7 = x.main' * tmp4;
        
        Dgf = (tmp4 * tmp2') + tmp1 * (tmp6' + tmp7') + (tmp5 * tmp2) + tmp3 * (tmp6 + tmp7);
        
        if(i == 1 || i == dl)
            Q2 = Q2 + ((b - a) / (N1-1)) * (Dgf / 2);
        else
            Q2 = Q2 + ((b - a) / (N1-1)) * Dgf;
        end
    end
    
    a = lag(dl); b = lag(du);
    N2 = length(dl : du);
    for i = dl : du
        tmp1 = C(:, :, i) * x.main;
        tmp2 = x.main' * tmp1;
        tmp3 = C(:, :, i)' * x.main;
        tmp4 = C(:, :, i) * eta.main;
        tmp5 = C(:, :, i)' * eta.main;
        tmp6 = eta.main' * tmp1;
        tmp7 = x.main' * tmp4;
        
        offDiag1 = tmp2 - diag(diag(tmp2));
        offDiag2 = tmp6 - diag(diag(tmp6));
        offDiag3 = tmp7 - diag(diag(tmp7));
        
        DgfO = (tmp4 * offDiag1') + (tmp5 * offDiag1) + tmp1 * (offDiag2' + offDiag3') + tmp3 * (offDiag2 + offDiag3);
%         DgfO = (tmp4 * (triu(tmp2,1) + tril(tmp2,-1))') + (tmp1 * (triu(tmp6,1) + tril(tmp6,-1))') + ...
%             (tmp1 * (triu(tmp7,1) + tril(tmp7,-1))') + (tmp5 * (triu(tmp2,1) + tril(tmp2,-1))) + ...
%             (tmp3 * (triu(tmp6,1) + tril(tmp6,-1))) + (tmp3 * (triu(tmp7,1) + tril(tmp7,-1)));

        if(i == dl || i == du)
            Q2 = Q2 + ((b - a) / (N2-1)) * (DgfO / 2);
        else
            Q2 = Q2 + ((b - a) / (N2-1)) * DgfO;
        end
    end
    
    a = lag(du); b = lag(N);
    N3 = length(du : N);
    for i = du : N
        tmp1 = C(:, :, i) * x.main;
        tmp2 = x.main' * tmp1;
        tmp3 = C(:, :, i)' * x.main;
        tmp4 = C(:, :, i) * eta.main;
        tmp5 = C(:, :, i)' * eta.main;
        tmp6 = eta.main' * tmp1;
        tmp7 = x.main' * tmp4;
        
        Dgf = (tmp4 * tmp2') + tmp1 * (tmp6' + tmp7') + (tmp5 * tmp2) + tmp3 * (tmp6 + tmp7);
        
        if(i == du || i == N)
            Q2 = Q2 + ((b - a) / (N3-1)) * (Dgf / 2);
        else
            Q2 = Q2 + ((b - a) / (N3-1)) * Dgf;
        end
    end 
    output.main = 2 * Q2;
end


%% Generates m Prolate Spheroidal Wave Functions in time-bandwidth T and W 
% evaluated at x.
%
% psi = pswf(x, T, W, m, eps)
% Input:
%   x   - A 1 by k size vector of evaluation points. Values must be between 
%         -1 and 1.
%   T   - Time limit of functions to be generated.
%   W   - Bandwidth of functions to be generated.
%   m   - (optional) Number of basis functions to return.
%   eps - (optional) Min precision of computed pswfs.
% 
% Output:
%   psi - Prolate Spheroidal wave functions. A m by k matrix where every
%         row is a pswf.
% 
% Reference:
%   H. Xiao, ``Prolate Spheroidal Wave Functions, Quadrature,
%   Interpolation, and Asymptotic Formulae.'' PhD thesis, Yale University,
%   2001, pp. 19-21.
function psi = pswf(x, T, W , m, eps)
    if nargin<5
        eps = 1e-4;
    end
    c = pi*T*W;
    if nargin<4
        n = ceil(2*(exp(1)*c+1)+log2(1/eps));
        m = n;
    else
        n = max(ceil(2*(exp(1)*c+1)+log2(1/eps)), m);
    end
    
    t = (0:n).';
    am1 = (t.*(t-1))./((2*t-1).*sqrt((2*t-3).*(2*t+1)))*c^2;
    a0 = t.*(t+1)+(2*t.*(t+1)-1)./((2*t+3).*(2*t-1))*c^2;
    a1 = ((t+2).*(t+1))./((2*t+3).*sqrt((2*t+1).*(2*t+5)))*c^2;

    H = spdiags([a1 a0 am1], [-2 0 2], n+1, n+1);
    H = H/n^2;
    
    [U, e] = eig(full(H));
    [~, jj] = sort(diag(e));
    U = U(:, jj);
    
    P = legendrePoly(n-1,x);
    
    psi = U(1:n,1:m)'*P;
    for k=1:size(psi,1)
        psi(k,:) = psi(k,:)/norm(psi(k,:));
    end
end

%% Compute the first n normalized Legendre polynomials
% 
% P = legendrePoly(n, X)
% Input:
%   n - Legendre polynomial to compute up to. (n>=0)
%   X - Evaluation points for Legendre polynomials. Must be a 1xm vector 
%       with values between -1 and 1.
% Output:
%   P - First n normalized Legendre polynomials. Is a n by m matrix where
%       every right is a Legendre polynomial evaluated at X.
% 
% Reference:
%   H. Xiao, ``Prolate Spheroidal Wave Functions, Quadrature,
%   Interpolation, and Asymptotic Formulae.'' PhD thesis, Yale University,
%   2001, pp. 5-6.
function P = legendrePoly(n, X)
    if n<0
        error('n must be non-negative');
    end
    P = zeros(n+1, length(X));
    
    P(1,:) = ones(1,length(X));
    if n==0
        return;
    end
    
    P(2,:) = X;
    if n==1
        return;
    end
    
    for k=2:n
        P(k+1,:) = (2*k-1)*X.*P(k,:) - (k-1)*P(k-1,:);
        P(k+1,:) = P(k+1,:)/k;
    end
    
    % normalize them
    for k=0:n
        P(k+1,:) = P(k+1,:)*sqrt(k+0.5);
    end
end

%% FFT based signal time shift
% 
% f = fourierShift(f, d, dt)
% Input:
%   f  - row vector of signal to be shifted. If f is a matrix each row is
%        shifted.
%   d  - desired delay
%   dt - sample spacing
% 
% Output:
%   f  - time shifted signal
function f = fourierShift(f, d, dt)
    colvec = 0;
    
    if isvector(f) && size(f,1)~=1
        colvec = 1;
        f = f';
    end
    
    [m,n] = size(f);
    if isreal(f)
        realVal = 1;
    else
        realVal = 0;
    end
    
    f = fft(f,[],2);
    f = fftshift(f,2);
    
    w = (-ceil((n-1)/2):floor((n-1)/2))/dt/n;
    f = f.*exp(-2*pi*1i*d*w(ones(1,m),:));
    
    f = ifftshift(f,2);
    f = ifft(f,[],2);
    
    if realVal
        f = real(f);
    end
    
    dn = ceil(abs(d)/dt);
    ix = (1:n);
    if d>0
        f = f.*(ix(ones(1,m),:)>dn);
    elseif d<0
        
        f = f.*(ix(ones(1,m),:)<n-dn);
    end
    
    if colvec
        f = f';
    end
end

%% Generate Linear Frequency Modulated Signals
% 
% [s1, s2] = mimo_lfm(T, W, dt)
% Input:
%   T - time period
%   W - frequency
%   dt - (optional) time samples
% Output:
%   s1, s2 - 2 LFM signals
function [s1, s2] = mimo_lfm(T, W, dt)
    opt.Display = 0;
    T = 0.002;
    W = 30000;
    
    if nargin < 3
        dt = 1/(4*W); %mean(diff(x));
    end
    x = 0:dt:T;
    s1 = chirp(x, 0, T, W);
    s1 = s1.*tukeywin(length(s1), 0.2).';
    s2 = fliplr(s1);
    
    s1 = s1/norm(s1);
    s2 = s2/norm(s2);
    
    if opt.Display
        [n, m] = size(s1);
        
        figure(1)
        [ccf, clags] = xcorr(s1, s2, floor(m/2));
        plot(clags*dt,20*log10(abs(ccf)))
        axis([-T/2 T/2 -30 0])
        title('cross-correlation')

        figure(2)
        [acf2, aclags] = xcorr(s1, floor(m/2));
        plot(aclags*dt,20*log10(abs(acf2)), '--r')
        axis([-T/2 T/2 -30 0])
        title('auto-correlation of s_1')

        figure(3)
        [acf2, aclags] = xcorr(s2, floor(m/2));
        plot(aclags*dt,20*log10(abs(acf2)), '--r')
        axis([-T/2 T/2 -30 0])
        title('auto-correlation of s_2')
        
        
        time = (1:m)*dt;
        figure(4)
        hs1 = subplot(2, 1, 1);
        plot(time, s1, 'r')
        set(get(gcf,'CurrentAxes'),'fontsize',14)
        xlabel('time','fontsize',14); ylabel('s_1','fontsize',14)
        
        hs2 = subplot(2, 1, 2);
        plot(time, s2, 'r')
        set(get(gcf,'CurrentAxes'),'fontsize',14)
        xlabel('time','fontsize',14); ylabel('s_2','fontsize',14)
        ylb = min(min(s1, s2));
        yub = max(max(s1, s2));
        axis([hs1 hs2], [time(1) time(m) ylb yub]);
    end
end